@@ -25,7 +25,10 @@ void variant_file::output_as_plink(const parameters &params)
 		if (include_indv[ui] == false)
 			continue;
 
-		char tmpname[] = "/tmp/vcftools.XXXXXX";
+		string new_tmp = params.temp_dir+"/vcftools.XXXXXX";
+		char tmpname[new_tmp.size()];
+		strcpy(tmpname, new_tmp.c_str());
+
 		fd = mkstemp(tmpname);
 		if (fd == -1)
 			LOG.error(" Could not open temporary file.\n", 12);
@@ -47,23 +50,43 @@ void variant_file::output_as_plink(const parameters &params)
 	int POS;
 	string ID, CHROM, CHROM2;
 	map<string, string> CHROM_to_PLINK;
-	for (int i=1; i<23; i++)
+
+	if (params.chrom_map_file != "")
 	{
-		ostringstream convert;
-		convert << i;
-		CHROM_to_PLINK["chr" + convert.str()] = convert.str();
-		CHROM_to_PLINK[convert.str()] = convert.str();
+		ifstream chrom_map(params.chrom_map_file.c_str());
+		if (!chrom_map.is_open())
+			LOG.error("Could not open chromosome mapping file: " + params.chrom_map_file);
+		string chr, plink;
+		unsigned int N_chrom_entries=0;
+		while (!chrom_map.eof())
+		{
+			chrom_map >> chr >> plink;
+			CHROM_to_PLINK[chr] = plink;
+			N_chrom_entries++;
+		}
+		chrom_map.close();
+		LOG.printLOG("\tRead " + output_log::int2str(N_chrom_entries) + " chromosome mapping file entries.\n");
+	}
+	else
+	{
+		for (int i=1; i<23; i++)
+		{
+			ostringstream convert;
+			convert << i;
+			CHROM_to_PLINK["chr" + convert.str()] = convert.str();
+			CHROM_to_PLINK[convert.str()] = convert.str();
+		}
+		CHROM_to_PLINK["chrX"] = "X";
+		CHROM_to_PLINK["chrY"] = "Y";
+		CHROM_to_PLINK["chrXY"] = "XY";
+		CHROM_to_PLINK["chrMT"] = "MT";
+		CHROM_to_PLINK["chrM"] = "M";
+		CHROM_to_PLINK["X"] = "X";
+		CHROM_to_PLINK["Y"] = "Y";
+		CHROM_to_PLINK["XY"] = "XY";
+		CHROM_to_PLINK["MT"] = "MT";
+		CHROM_to_PLINK["M"] = "M";
 	}
-	CHROM_to_PLINK["chrX"] = "X";
-	CHROM_to_PLINK["chrY"] = "Y";
-	CHROM_to_PLINK["chrXY"] = "XY";
-	CHROM_to_PLINK["chrMT"] = "MT";
-	CHROM_to_PLINK["chrM"] = "M";
-	CHROM_to_PLINK["X"] = "X";
-	CHROM_to_PLINK["Y"] = "Y";
-	CHROM_to_PLINK["XY"] = "XY";
-	CHROM_to_PLINK["MT"] = "MT";
-	CHROM_to_PLINK["M"] = "M";
 
 	vector<string> alleles;
 	char phase;
@@ -109,7 +132,7 @@ void variant_file::output_as_plink(const parameters &params)
 				CHROM_to_PLINK[CHROM] = tmp;
 			else
 			{
-				LOG.one_off_warning("\nUnrecognized values used for CHROM: " + CHROM + " - Replacing with 0.\n");
+				LOG.one_off_warning("\nUnrecognized values used for CHROM: " + CHROM + " - Replacing with 0.");
 				CHROM_to_PLINK[CHROM] = "0";
 			}
 		}
@@ -140,6 +163,8 @@ void variant_file::output_as_plink(const parameters &params)
 
 			if (genotype.first < 0)
 				(*tmp_file) << "\t0";
+			else if (genotype.first > 1)
+				LOG.error("File contains entries with nonexistent genotypes at " + CHROM + ":" + output_log::int2str(e->get_POS()) );
 			else
 				(*tmp_file) << "\t" << alleles[genotype.first];
 
@@ -152,6 +177,8 @@ void variant_file::output_as_plink(const parameters &params)
 				else
 					(*tmp_file) << "\t0";
 			}
+			else if (genotype.second > 1)
+				LOG.error("File contains entries with nonexistent genotypes at " + CHROM + ":" + output_log::int2str(e->get_POS()) );
 			else
 				(*tmp_file) << "\t" << alleles[genotype.second];
 		}
@@ -204,21 +231,42 @@ void variant_file::output_as_plink_tped(const parameters &params)
 
 	string CHROM, CHROM2;
 	map<string, string> CHROM_to_PLINK;
-	for (int i=1; i<23; i++)
+	if (params.chrom_map_file != "")
 	{
-		ostringstream convert;
-		convert << i;
-		CHROM_to_PLINK["chr" + convert.str()] = convert.str();
-		CHROM_to_PLINK[convert.str()] = convert.str();
+		ifstream chrom_map(params.chrom_map_file.c_str());
+		if (!chrom_map.is_open())
+			LOG.error("Could not open chromosome mapping file: " + params.chrom_map_file);
+		string chr, plink;
+		unsigned int N_chrom_entries=0;
+		while (!chrom_map.eof())
+		{
+			chrom_map >> chr >> plink;
+			CHROM_to_PLINK[chr] = plink;
+			N_chrom_entries++;
+		}
+		chrom_map.close();
+		LOG.printLOG("\n\tRead " + output_log::int2str(N_chrom_entries) + " chromosome mapping file entries.\n");
+	}
+	else
+	{
+		for (int i=1; i<23; i++)
+		{
+			ostringstream convert;
+			convert << i;
+			CHROM_to_PLINK["chr" + convert.str()] = convert.str();
+			CHROM_to_PLINK[convert.str()] = convert.str();
+		}
+		CHROM_to_PLINK["chrX"] = "X";
+		CHROM_to_PLINK["chrY"] = "Y";
+		CHROM_to_PLINK["chrXY"] = "XY";
+		CHROM_to_PLINK["chrMT"] = "MT";
+		CHROM_to_PLINK["chrM"] = "M";
+		CHROM_to_PLINK["X"] = "X";
+		CHROM_to_PLINK["Y"] = "Y";
+		CHROM_to_PLINK["XY"] = "XY";
+		CHROM_to_PLINK["MT"] = "MT";
+		CHROM_to_PLINK["M"] = "M";
 	}
-	CHROM_to_PLINK["chrX"] = "X";
-	CHROM_to_PLINK["chrY"] = "Y";
-	CHROM_to_PLINK["chrXY"] = "XY";
-	CHROM_to_PLINK["chrMT"] = "MT";
-	CHROM_to_PLINK["X"] = "X";
-	CHROM_to_PLINK["Y"] = "Y";
-	CHROM_to_PLINK["XY"] = "XY";
-	CHROM_to_PLINK["MT"] = "MT";
 
 	vector<string> alleles;
 	char phase;
@@ -259,7 +307,7 @@ void variant_file::output_as_plink_tped(const parameters &params)
 				CHROM_to_PLINK[CHROM] = tmp;
 			else
 			{
-				LOG.one_off_warning("\nUnrecognized values used for CHROM: " + CHROM + " - Replacing with 0.\n");
+				LOG.one_off_warning("\nUnrecognized values used for CHROM: " + CHROM + " - Replacing with 0.");
 				CHROM_to_PLINK[CHROM] = "0";
 			}
 		}
@@ -288,6 +336,8 @@ void variant_file::output_as_plink_tped(const parameters &params)
 
 			if (genotype.first < 0)
 				TPED << "\t0";
+			else if (genotype.first > 1)
+				LOG.error("File contains entries with nonexistent genotypes at " + CHROM + ":" + output_log::int2str(e->get_POS()) );
 			else
 				TPED << "\t" << alleles[genotype.first];
 
@@ -300,6 +350,8 @@ void variant_file::output_as_plink_tped(const parameters &params)
 				else
 					TPED << "\t0";
 			}
+			else if (genotype.second > 1)
+				LOG.error("File contains entries with nonexistent genotypes at " + CHROM + ":" + output_log::int2str(e->get_POS()) );
 			else
 				TPED << "\t" << alleles[genotype.second];
 		}
@@ -347,7 +399,10 @@ void variant_file::output_as_012_matrix(const parameters &params)
 			continue;
 		FAM << meta_data.indv[ui] << endl;
 
-		char tmpname[] = "/tmp/vcftools.XXXXXX";
+		string new_tmp = params.temp_dir+"/vcftools.XXXXXX";
+		char tmpname[new_tmp.size()];
+		strcpy(tmpname, new_tmp.c_str());
+
 		fd = mkstemp(tmpname);
 		if (fd == -1)
 			LOG.error(" Could not open temporary file.\n", 12);
@@ -567,14 +622,12 @@ void variant_file::output_as_LDhat_phased(const parameters &params)
 
 	unsigned int n_sites = 0;
 	int max_pos = -1;
-	int fd = -1;
-
-	char tmpname[] = "/tmp/vcftools.XXXXXX";
-	fd = mkstemp(tmpname);
-	if (fd == -1)
-		LOG.error(" Could not open temporary file.\n", 12);
-	::close(fd);
+	char* fd;
 
+	string new_tmp = params.temp_dir+"/vcftools.XXXXXX";
+	char tmpname[new_tmp.size()];
+	strcpy(tmpname, new_tmp.c_str());
+	fd = mktemp(tmpname);
 	string locs_tmp_filename(tmpname);
 	ofstream *locs_tmp_file = new ofstream(tmpname);
 	if (!locs_tmp_file->good())
@@ -600,29 +653,46 @@ void variant_file::output_as_LDhat_phased(const parameters &params)
 		if (include_indv[ui] == false)
 			continue;
 
-		char tmpname[] = "/tmp/vcftools.XXXXXX";
-		fd = mkstemp(tmpname);
-		if (fd == -1)
-			LOG.error(" Could not open temporary file.\n", 12);
-		::close(fd);
+		char tmpname[new_tmp.size()];
+		strcpy(tmpname, new_tmp.c_str());
+		fd = mktemp(tmpname);
 
 		ofstream *tmp_file = new ofstream(tmpname);
 		if (!tmp_file->good())
-			LOG.error("Could not open temp file #" + output_log::int2str(ui) + ".\n", 12);
+		{	// Clean up temp files.
+			tmp_file->close(); remove(tmpname);
+			locs_tmp_file->close(); remove(locs_tmp_filename.c_str());
+			for (unsigned int uj=0; uj<ui; uj++)
+			{
+				(tmp_files[2*uj])->close(); remove(tmp_filenames[2*uj].c_str());
+				(tmp_files[2*uj+1])->close(); remove(tmp_filenames[2*uj+1].c_str());
+			}
+			LOG.error("\n\nCould not open temporary file.\n\n"
+					"Most likely this is because the system is not allowing me to open enough temporary files.\n"
+					"Try using ulimit -n <int> to increase the number of allowed open files.\n", 12);
+		}
 		tmp_files[2*ui] = tmp_file;
 		tmp_filenames[2*ui] = tmpname;
 
-		char tmpname2[] = "/tmp/vcftools.XXXXXX";
-		fd = mkstemp(tmpname2);
-		if (fd == -1)
-			LOG.error(" Could not open temporary file.\n", 12);
-		::close(fd);
+		char tmpname2[new_tmp.size()];
+		strcpy(tmpname2, new_tmp.c_str());
+		fd = mktemp(tmpname2);
 
 		ofstream *tmp_file2 = new ofstream(tmpname2);
 		if (!tmp_file2->good())
+		{	// Clean up temp files.
+			tmp_file2->close(); remove(tmpname2);
+			locs_tmp_file->close(); remove(locs_tmp_filename.c_str());
+			for (unsigned int uj=0; uj<ui; uj++)
+			{
+				(tmp_files[2*uj])->close(); remove(tmp_filenames[2*uj].c_str());
+				(tmp_files[2*uj+1])->close(); remove(tmp_filenames[2*uj+1].c_str());
+			}
+			(tmp_files[2*ui])->close(); remove(tmp_filenames[2*ui].c_str());
 			LOG.error("\n\nCould not open temporary file.\n\n"
 				"Most likely this is because the system is not allowing me to open enough temporary files.\n"
 				"Try using ulimit -n <int> to increase the number of allowed open files.\n", 12);
+		}
 		tmp_files[2*ui+1] = tmp_file2;
 		tmp_filenames[2*ui+1] = tmpname2;
 	}
@@ -669,7 +739,7 @@ void variant_file::output_as_LDhat_phased(const parameters &params)
 				else
 					geno = alleles.second;
 
-				if ((geno > 0) && (e->include_genotype[ui]==true))
+				if ((geno >= 0) && (e->include_genotype[ui]==true))
 					(*tmp_file) << geno;
 				else
 					(*tmp_file) << "?";
@@ -732,14 +802,12 @@ void variant_file::output_as_LDhat_unphased(const parameters &params)
 
 	unsigned int n_sites = 0;
 	int max_pos = -1;
-	int fd = -1;
-
-	char tmpname[] = "/tmp/vcftools.XXXXXX";
-	fd = mkstemp(tmpname);
-	if (fd == -1)
-		LOG.error(" Could not open temporary file.\n", 12);
-	::close(fd);
+	char* fd;
 
+	string new_tmp = params.temp_dir+"/vcftools.XXXXXX";
+	char tmpname[new_tmp.size()];
+	strcpy(tmpname, new_tmp.c_str());
+	fd = mktemp(tmpname);
 	string locs_tmp_filename(tmpname);
 	ofstream *locs_tmp_file = new ofstream(tmpname);
 	if (!locs_tmp_file->good())
@@ -764,18 +832,24 @@ void variant_file::output_as_LDhat_unphased(const parameters &params)
 	{
 		if (include_indv[ui] == false)
 			continue;
-		char tmpname[] = "/tmp/vcftools.XXXXXX";
-		fd = mkstemp(tmpname);
-		if (fd == -1)
-			LOG.error(" Could not open temporary file.\n", 12);
-		::close(fd);
 
+		char tmpname[new_tmp.size()];
+		strcpy(tmpname, new_tmp.c_str());
+		fd = mktemp(tmpname);
 		string filename(tmpname);
 		ofstream *tmp_file = new ofstream(tmpname);
 		if (!tmp_file->good())
+		{	// Clean up temp files
+			tmp_file->close(); remove(tmpname);
+			locs_tmp_file->close(); remove(locs_tmp_filename.c_str());
+			for (unsigned int uj=0; uj<ui; uj++)
+			{
+				(tmp_files[uj])->close(); remove(tmp_filenames[uj].c_str());
+			}
 			LOG.error("\n\nCould not open temporary file.\n\n"
 					"Most likely this is because the system is not allowing me to open enough temporary files.\n"
 					"Try using ulimit -n <int> to increase the number of allowed open files.\n", 12);
+		}
 		tmp_files[ui] = tmp_file;
 		tmp_filenames[ui] = filename;
 	}
